==> ump-0.8.3.2-src.tar.bz2: ump-0.8.3.2/src/math2.array.cpp <==


/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See math2.h
 */


#include "math2.h"

namespace math
{

Array::Array()
{
	nrofvariables = 0;
	array = 0;
}
Array::Array(const Array &src) throw(error_obj)
{
	*this = Array();
	*this = src;
}
Array::~Array()
{
	clear();
}

bool Array::operator==(const Array &arr) const
{
	if( nrofvariables != arr.nrofvariables )
		return false;
	
	for( uint32 i=0; i<nrofvariables; i++ )
	{
		if( *array[i] != *arr.array[i] )
			return false;
	}
	return true;
}

void Array::clear(void)
{
	for(uint32 i=0;i<nrofvariables;i++)
		delete array[i];
	delete [] array;

	array = 0;
	nrofvariables = 0;
}

void Array::operator=(const Array &src) throw(error_obj)
{
	clear();

	try
	{
		for(uint32 i=0;i<src.nrofvariables;i++)
			add_variable(src.array[i]);
	}
	catch(...)
	{
		clear();
		throw;
	}
}

void Array::add_this_variable( Variable *var ) throw(error_obj)
{
	Variable  **newarray = 0;
	if(var == 0)
		return;

	try
	{
		newarray = new Variable * [nrofvariables + 1];
		newarray[nrofvariables] = var;
	}
	catch(...)
	{
		if(newarray != 0)
			delete newarray[nrofvariables];
		delete [] newarray;
		Throw_Couldnt_get_memory();
	}

	for(uint32 i=0;i<nrofvariables;i++)
		newarray[i] = array[i];

	delete [] array;

	nrofvariables++;

	array = newarray;
}

void Array::add_variable(const Variable *var) throw(error_obj)
{
	Variable *new_var;
	if(var == 0)
		return;

	try
	{
		new_var = 0;
		new_var = new Variable;
		*new_var = *var;
		new_var->set_name_from( *var );
    
		add_this_variable( new_var );
	}
	catch(...)
	{
		delete new_var;
		Throw_Couldnt_get_memory();
	}
}

const Variable * Array::get_variable(uint32 nr) const throw(error_obj)
{
	if(nr >= nrofvariables)
		THROW_MACRO_FIX( 0, "Array-index out of range" )
	return array[nr];
}

void Array::set_variable( uint32 nr, const Variable &new_var) throw(error_obj)
{
	if(nr >= nrofvariables)
		THROW_MACRO_FIX( 0, "Array-index out of range" )
  
	  *array[nr] = new_var;
	  array[nr]->set_name_from( new_var );
}

void Array::set_size( uint32 new_size ) throw(error_obj)
{
	clear();

	try
	{
		array = new Variable * [new_size];
		nrofvariables = new_size;
    
		for( uint32 i=0; i<new_size; i++ )
			array[i] = 0;
    
		for( uint32 i=0; i<new_size; i++ )
			array[i] = new Variable;
	}
	catch(...)
	{
		clear();
		Throw_Couldnt_get_memory();
	}
}

void Array::change_size_to( uint32 new_size ) throw(error_obj)
{
	Variable **new_array = 0;
	uint32 i;

	if( new_size == nrofvariables )
		return;

	try{  new_array = new Variable * [new_size + 1];  }
	catch(...) {  Throw_Couldnt_get_memory();  }

	for( i=0; i<nrofvariables  &&  i<new_size; i++ )
		new_array[i] = array[i];

	if( nrofvariables < new_size )
	{
		for( i=nrofvariables; i<new_size; i++ )
			new_array[i] = 0;

		try
		{
			for( i=nrofvariables; i<new_size; i++ )
			new_array[i] = new Variable;
		}
		catch( ... )
		{
			delete [] array;
			array = new_array;
			clear();
			Throw_Couldnt_get_memory();
		}
	}
	else //if( nrofvariables > new_size )
	{
		for( i=new_size; i<nrofvariables; i++ )
		delete array[i];
	}
	delete [] array;
	nrofvariables = new_size;;
	array = new_array;
}


void Array::insert( const Variable *var, uint32 at_pos ) throw(error_obj)
{
	Variable *new_var;
	if(var == 0)
		return;

	try
	{
		new_var = 0;
		new_var = new Variable;
		*new_var = *var;
		new_var->set_name_from( *var );
    
		insert_this( new_var, at_pos );
	}
	catch(...)
	{
		delete new_var;
		Throw_Couldnt_get_memory();
	}
}

void Array::insert_this( Variable *var, uint32 at_pos ) throw(error_obj)
{
	if( at_pos > nrofvariables )
		at_pos = nrofvariables;

	change_size_to( nrofvariables + 1 );

	for( uint32 i=nrofvariables-1; i>at_pos  &&  i!=uint32_max; i-- )
		array[i] = array[i-1];

	array[at_pos] = var;
}
  
void Array::remove_range( uint32 from_nr, uint32 this_many )
{
	if( this_many == 0  ||  from_nr >= nrofvariables )
		return;
  
	if( (from_nr + this_many) >= nrofvariables )
	{
		for( uint32 i=from_nr; i<nrofvariables; i++ )
			delete array[i];
		nrofvariables = from_nr;
	}
	else
	{
		uint32 removed=0;
		nrofvariables -= this_many;
		for( uint32 i=from_nr; i<nrofvariables; i++ )
		{
			if( removed != this_many )
			{
				delete array[i];
				removed++;
			}
			array[i] = array[i+this_many];
		}
	}
}



} // namespace math

==> ump-0.8.4-src.tar.bz2: ump-0.8.4/src/math2.array.cpp <==


/*
 *  Math2
 *  Copyright (c) 2003-2006 by Mattias Hultgren <mattias_hultgren@tele2.se>
 *
 *  See math2.h
 */


#include "math2.h"

namespace math
{

Array::Array()
{
	nrofvariables = 0;
	array = 0;
}
Array::Array(const Array &src) throw(error_obj)
{
	nrofvariables = 0;
	array = 0;
	*this = src;
}
Array::~Array()
{
	clear();
}

bool Array::operator==(const Array &arr) const
{
	if( nrofvariables != arr.nrofvariables )
		return false;

	for( uint32 i=0; i<nrofvariables; i++ )
	{
		if( *array[i] != *arr.array[i] )
			return false;
	}
	return true;
}

void Array::clear(void)
{
	for(uint32 i=0;i<nrofvariables;i++)
		delete array[i];
	delete [] array;

	array = 0;
	nrofvariables = 0;
}

void Array::operator=(const Array &src) throw(error_obj)
{
	clear();

	try
	{
		for(uint32 i=0;i<src.nrofvariables;i++)
			add_variable(src.array[i]);
	}
	catch(...)
	{
		clear();
		throw;
	}
}

void Array::add_this_variable( Variable *var ) throw(error_obj)
{
	Variable  **newarray = 0;
	if(var == 0)
		return;

	try
	{
		newarray = new Variable * [nrofvariables + 1];
		newarray[nrofvariables] = var;
	}
	catch(...)
	{
		if(newarray != 0)
			delete newarray[nrofvariables];
		delete [] newarray;
		Throw_Couldnt_get_memory();
	}

	for(uint32 i=0;i<nrofvariables;i++)
		newarray[i] = array[i];

	delete [] array;

	nrofvariables++;

	array = newarray;
}

void Array::add_variable(const Variable *var) throw(error_obj)
{
	Variable *new_var;
	if(var == 0)
		return;

	try
	{
		new_var = 0;
		new_var = new Variable;
		*new_var = *var;
		new_var->set_name_from( *var );

		add_this_variable( new_var );
	}
	catch(...)
	{
		delete new_var;
		Throw_Couldnt_get_memory();
	}
}

const Variable * Array::get_variable(uint32 nr) const throw(error_obj)
{
	if(nr >= nrofvariables)
		THROW_MACRO_FIX( 0, "Array-index out of range" );
	return array[nr];
}

void Array::set_variable( uint32 nr, const Variable &new_var) throw(error_obj)
{
	if(nr >= nrofvariables)
		THROW_MACRO_FIX( 0, "Array-index out of range" )

	*array[nr] = new_var;
	array[nr]->set_name_from( new_var );
}

void Array::set_size( uint32 new_size ) throw(error_obj)
{
	clear();

	try
	{
		array = new Variable * [new_size];
		nrofvariables = new_size;

		for( uint32 i=0; i<new_size; i++ )
			array[i] = 0;

		for( uint32 i=0; i<new_size; i++ )
			array[i] = new Variable;
	}
	catch(...)
	{
		clear();
		Throw_Couldnt_get_memory();
	}
}

void Array::increase_size(void) throw(error_obj)
{
	Variable **new_array = 0;
	uint32 i;

	try{  new_array = new Variable * [nrofvariables + 1];  }
	catch(...) {  Throw_Couldnt_get_memory();  }

	for( i=0; i<nrofvariables; i++ )
		new_array[i] = array[i];

	new_array[nrofvariables] = 0;

	delete [] array;
	nrofvariables++;
	array = new_array;
}


void Array::insert( const Variable *var, uint32 at_pos ) throw(error_obj)
{
	Variable *new_var;
	if(var == 0)
		return;

	try
	{
		new_var = 0;
		new_var = new Variable;
		*new_var = *var;
		new_var->set_name_from( *var );

		insert_this( new_var, at_pos );
	}
	catch(...)
	{
		delete new_var;
		Throw_Couldnt_get_memory();
	}
}

void Array::insert_this( Variable *var, uint32 at_pos ) throw(error_obj)
{
	if( at_pos > nrofvariables )
		at_pos = nrofvariables;

	increase_size();

	for( uint32 i=nrofvariables-1; i>at_pos; i-- )
		array[i] = array[i-1];

	array[at_pos] = var;
}

void Array::remove_range( uint32 from_nr, uint32 this_many )
{
	if( this_many == 0  ||  from_nr >= nrofvariables )
		return;

	if( (from_nr + this_many) >= nrofvariables )
	{
		for( uint32 i=from_nr; i<nrofvariables; i++ )
			delete array[i];
		nrofvariables = from_nr;
	}
	else
	{
		uint32 removed=0;
		nrofvariables -= this_many;
		for( uint32 i=from_nr; i<nrofvariables; i++ )
		{
			if( removed != this_many )
			{
				delete array[i];
				removed++;
			}
			array[i] = array[i+this_many];
		}
	}
}



} // namespace math
